#include "item.h"
#include "xpm/item.h"

BirdItem::BirdItem(){
  CreateHeartItem();
  CreateBigItem();
  CreateWideItem();
  CreateLineItem();
  CreateCrossItem();
  CreateXRevItem();
  CreateWaveItem();
  CreateHomingItem();
  CreateDivideItem();
  CreateDropItem();
  CreateBoostItem();
  CreateSpeedItem();
  CreatePassItem();
}

BirdItem::~BirdItem(){
  delete itemBox[ITEM_HEART];
  delete itemBox[ITEM_BIG];
  delete itemBox[ITEM_WIDE];
  delete itemBox[ITEM_LINE];
  delete itemBox[ITEM_CROSS];
  delete itemBox[ITEM_XREV];
  delete itemBox[ITEM_WAVE];
  delete itemBox[ITEM_HOM];
  delete itemBox[ITEM_DIVIDE];
  delete itemBox[ITEM_DROP];
  delete itemBox[ITEM_BOOST];
  delete itemBox[ITEM_SPEED];
  delete itemBox[ITEM_PASS];
}

void BirdItem::Create(Glib::RefPtr<Gdk::Pixmap> frame0,
		      Glib::RefPtr<Gdk::GC> mGC0,
		      BirdMap *Map0,
		      BirdPlayer *player0){
  Map=Map0;
  frame=frame0;
  mGC=mGC0;
  player=player0;
}

void BirdItem::Appear(int px,int py,int width,int height,
		       int style,int touch,int tcount){
  BirdActor *act;
  act= new BirdActor();
  act->px=px;
  act->py=py;
  act->width=width;
  act->height=height;
  act->mx=0;
  act->my=0;
  act->life=10;  
  act->style=style;
  act->move=0;
  act->touch=touch;
  act->tcount=tcount;

  act->dcount=0;
  act->direct=0;
  act->seq=0;

  act->next=NULL;
  act->prev=NULL;
  Append(act); 
}

int BirdItem::Animate(){
  BirdActor *node;
  BirdActor *tmp;
  SpriteBox *spriteBox0;
  int i;
  int playerFlag=0;

  i=0;
  node=first;
  while(node){
    tmp=node->next;
    spriteBox0=itemBox[node->style];

    if(node->life>0){

      node->px+=Map->speedX;
      node->py+=Map->speedY;

      playerFlag=ContactRect(player,node);
      (this->*ITEM_TOUCH[node->touch])(node,playerFlag);
      
      if(node->px<0){
	node->dcount=0;
	node->life=0;
      }
      else if(node->px>WIDTH){
	node->dcount=0;
	node->life=0;	
      }
      if(node->py<0){
	node->dcount=0;
	node->life=0;
      }
      else if(node->py>HEIGHT){
	node->dcount=0;
	node->life=0;
      }

    }

    if(node->life>0){
      node->seq++;
      if(node->seq>=spriteBox0->nSeq){
	node->seq=0;
      }
      spriteBox0->DrawSprite(frame,mGC,node->px,node->py,
			     node->seq+node->direct*spriteBox0->nSeq);
    }
    else{
      Reject(node);
      delete node;
    }

    node=tmp;
    i++;
  }
  return i;
}

void BirdItem::TouchAddLife(BirdActor *act,int playerFlag){
  if(playerFlag){
    player->life++;
    act->life=0;
  }
}
void BirdItem::TouchChangeBigFire(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->fire=PF_BIG;
   act->life=0;
 }
}
void BirdItem::TouchChangeWideFire(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->fire=PF_WIDE;
   act->life=0;
 }
}
void BirdItem::TouchChangeLineFire(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->fire=PF_LINE;
   act->life=0;
 }
}
void BirdItem::TouchChangeCrossForm(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->form=PLAYER_FORM_CROSS;
   act->life=0;
 }
}
void BirdItem::TouchChangeXRevForm(BirdActor *act,int playerFlag){
   if(playerFlag){
     player->form=PLAYER_FORM_XREV;
     act->life=0;
   }
}
void BirdItem::TouchChangeWaveForm(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->form=PLAYER_FORM_WAVE;
   act->life=0;
 }
}
void BirdItem::TouchChangeHomingForm(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->form=PLAYER_FORM_HOM;
   act->life=0;
 }
}
void BirdItem::TouchChangeDivideForm(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->form=PLAYER_FORM_DIVIDE;
   act->life=0;
 }
}
void BirdItem::TouchChangeDropForm(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->form=PLAYER_FORM_DROP;
   act->life=0;
 }
}
void BirdItem::TouchChangeBoostMode(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->item=PLAYER_MODE_BOOST;
   player->SetMode(PLAYER_MODE_BOOST);
   act->life=0;
 }
}
void BirdItem::TouchChangeSpeedMode(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->item=PLAYER_MODE_SPEED;
   player->SetMode(PLAYER_MODE_SPEED);
   act->life=0;
 }
}
void BirdItem::TouchChangePassMode(BirdActor *act,int playerFlag){
 if(playerFlag){
   player->item=PLAYER_MODE_PASS;
   player->SetMode(PLAYER_MODE_PASS);
   act->life=0;
 }
}

void (BirdItem::*BirdItem::ITEM_TOUCH[ITEM_TYPE_NUM])
  (BirdActor *act,int playerFlag)={
    &BirdItem::TouchAddLife,
    &BirdItem::TouchChangeBigFire,
    &BirdItem::TouchChangeWideFire,
    &BirdItem::TouchChangeLineFire,
    &BirdItem::TouchChangeCrossForm,
    &BirdItem::TouchChangeXRevForm,
    &BirdItem::TouchChangeWaveForm,
    &BirdItem::TouchChangeHomingForm,
    &BirdItem::TouchChangeDivideForm,
    &BirdItem::TouchChangeDropForm,
    &BirdItem::TouchChangeBoostMode,
    &BirdItem::TouchChangeSpeedMode,
    &BirdItem::TouchChangePassMode
};

void BirdItem::CreateHeartItem(){
  heartItem[0].xpm=(char **)heartitem_xpm;
  heartItem[1].xpm=(char **)heartitem_xpm;
  heartItem[2].xpm=(char **)heartitem_xpm;
  heartItem[3].xpm=(char **)heartitem_xpm;
  heartItem[4].xpm=(char **)heartitem_xpm;
  heartItem[5].xpm=(char **)heartitem_xpm;
  heartItem[6].xpm=(char **)heartitem_xpm;
  heartItem[7].xpm=(char **)heartitem_xpm;
  itemBox[ITEM_HEART]= new SpriteBox(heartItem,1,8);
  itemBox[ITEM_HEART]->SetSprite();
}

void BirdItem::CreateBigItem(){
  bigItem[0].xpm=(char **)bigitem_xpm;
  bigItem[1].xpm=(char **)bigitem_xpm;
  bigItem[2].xpm=(char **)bigitem_xpm;
  bigItem[3].xpm=(char **)bigitem_xpm;
  bigItem[4].xpm=(char **)bigitem_xpm;
  bigItem[5].xpm=(char **)bigitem_xpm;
  bigItem[6].xpm=(char **)bigitem_xpm;
  bigItem[7].xpm=(char **)bigitem_xpm;
  itemBox[ITEM_BIG]= new SpriteBox(bigItem,1,8);
  itemBox[ITEM_BIG]->SetSprite();
}

void BirdItem::CreateWideItem(){
  wideItem[0].xpm=(char **)wideitem_xpm;
  wideItem[1].xpm=(char **)wideitem_xpm;
  wideItem[2].xpm=(char **)wideitem_xpm;
  wideItem[3].xpm=(char **)wideitem_xpm;
  wideItem[4].xpm=(char **)wideitem_xpm;
  wideItem[5].xpm=(char **)wideitem_xpm;
  wideItem[6].xpm=(char **)wideitem_xpm;
  wideItem[7].xpm=(char **)wideitem_xpm;
  itemBox[ITEM_WIDE]= new SpriteBox(wideItem,1,8);
  itemBox[ITEM_WIDE]->SetSprite();
}

void BirdItem::CreateLineItem(){
  lineItem[0].xpm=(char **)lineitem_xpm;
  lineItem[1].xpm=(char **)lineitem_xpm;
  lineItem[2].xpm=(char **)lineitem_xpm;
  lineItem[3].xpm=(char **)lineitem_xpm;
  lineItem[4].xpm=(char **)lineitem_xpm;
  lineItem[5].xpm=(char **)lineitem_xpm;
  lineItem[6].xpm=(char **)lineitem_xpm;
  lineItem[7].xpm=(char **)lineitem_xpm;
  itemBox[ITEM_LINE]= new SpriteBox(lineItem,1,8);
  itemBox[ITEM_LINE]->SetSprite();
}

void BirdItem::CreateCrossItem(){
  crossItem[0].xpm=(char **)crossitem_xpm;
  crossItem[1].xpm=(char **)crossitem_xpm;
  crossItem[2].xpm=(char **)crossitem_xpm;
  crossItem[3].xpm=(char **)crossitem_xpm;
  crossItem[4].xpm=(char **)crossitem_xpm;
  crossItem[5].xpm=(char **)crossitem_xpm;
  crossItem[6].xpm=(char **)crossitem_xpm;
  crossItem[7].xpm=(char **)crossitem_xpm;
  itemBox[ITEM_CROSS]= new SpriteBox(crossItem,1,8);
  itemBox[ITEM_CROSS]->SetSprite();
}

void BirdItem::CreateXRevItem(){
  xrevItem[0].xpm=(char **)xrevitem_xpm;
  xrevItem[1].xpm=(char **)xrevitem_xpm;
  xrevItem[2].xpm=(char **)xrevitem_xpm;
  xrevItem[3].xpm=(char **)xrevitem_xpm;
  xrevItem[4].xpm=(char **)xrevitem_xpm;
  xrevItem[5].xpm=(char **)xrevitem_xpm;
  xrevItem[6].xpm=(char **)xrevitem_xpm;
  xrevItem[7].xpm=(char **)xrevitem_xpm;
  itemBox[ITEM_XREV]= new SpriteBox(xrevItem,1,8);
  itemBox[ITEM_XREV]->SetSprite();
}

void BirdItem::CreateWaveItem(){
  waveItem[0].xpm=(char **)waveitem_xpm;
  waveItem[1].xpm=(char **)waveitem_xpm;
  waveItem[2].xpm=(char **)waveitem_xpm;
  waveItem[3].xpm=(char **)waveitem_xpm;
  waveItem[4].xpm=(char **)waveitem_xpm;
  waveItem[5].xpm=(char **)waveitem_xpm;
  waveItem[6].xpm=(char **)waveitem_xpm;
  waveItem[7].xpm=(char **)waveitem_xpm;
  itemBox[ITEM_WAVE]= new SpriteBox(waveItem,1,8);
  itemBox[ITEM_WAVE]->SetSprite();
}

void BirdItem::CreateHomingItem(){
  homingItem[0].xpm=(char **)homingitem_xpm;
  homingItem[1].xpm=(char **)homingitem_xpm;
  homingItem[2].xpm=(char **)homingitem_xpm;
  homingItem[3].xpm=(char **)homingitem_xpm;
  homingItem[4].xpm=(char **)homingitem_xpm;
  homingItem[5].xpm=(char **)homingitem_xpm;
  homingItem[6].xpm=(char **)homingitem_xpm;
  homingItem[7].xpm=(char **)homingitem_xpm;
  itemBox[ITEM_HOM]= new SpriteBox(homingItem,1,8);
  itemBox[ITEM_HOM]->SetSprite();
}

void BirdItem::CreateDivideItem(){
  divideItem[0].xpm=(char **)divideitem_xpm;
  divideItem[1].xpm=(char **)divideitem_xpm;
  divideItem[2].xpm=(char **)divideitem_xpm;
  divideItem[3].xpm=(char **)divideitem_xpm;
  divideItem[4].xpm=(char **)divideitem_xpm;
  divideItem[5].xpm=(char **)divideitem_xpm;
  divideItem[6].xpm=(char **)divideitem_xpm;
  divideItem[7].xpm=(char **)divideitem_xpm;
  itemBox[ITEM_DIVIDE]= new SpriteBox(divideItem,1,8);
  itemBox[ITEM_DIVIDE]->SetSprite();
}

void BirdItem::CreateDropItem(){
  dropItem[0].xpm=(char **)dropitem_xpm;
  dropItem[1].xpm=(char **)dropitem_xpm;
  dropItem[2].xpm=(char **)dropitem_xpm;
  dropItem[3].xpm=(char **)dropitem_xpm;
  dropItem[4].xpm=(char **)dropitem_xpm;
  dropItem[5].xpm=(char **)dropitem_xpm;
  dropItem[6].xpm=(char **)dropitem_xpm;
  dropItem[7].xpm=(char **)dropitem_xpm;
  itemBox[ITEM_DROP]= new SpriteBox(dropItem,1,8);
  itemBox[ITEM_DROP]->SetSprite();
}

void BirdItem::CreateBoostItem(){
  boostItem[0].xpm=(char **)boostitem_xpm;
  boostItem[1].xpm=(char **)boostitem_xpm;
  boostItem[2].xpm=(char **)boostitem_xpm;
  boostItem[3].xpm=(char **)boostitem_xpm;
  boostItem[4].xpm=(char **)boostitem_xpm;
  boostItem[5].xpm=(char **)boostitem_xpm;
  boostItem[6].xpm=(char **)boostitem_xpm;
  boostItem[7].xpm=(char **)boostitem_xpm;
  itemBox[ITEM_BOOST]= new SpriteBox(boostItem,1,8);
  itemBox[ITEM_BOOST]->SetSprite();
}

void BirdItem::CreateSpeedItem(){
  speedItem[0].xpm=(char **)speeditem_xpm;
  speedItem[1].xpm=(char **)speeditem_xpm;
  speedItem[2].xpm=(char **)speeditem_xpm;
  speedItem[3].xpm=(char **)speeditem_xpm;
  speedItem[4].xpm=(char **)speeditem_xpm;
  speedItem[5].xpm=(char **)speeditem_xpm;
  speedItem[6].xpm=(char **)speeditem_xpm;
  speedItem[7].xpm=(char **)speeditem_xpm;
  itemBox[ITEM_SPEED]= new SpriteBox(speedItem,1,8);
  itemBox[ITEM_SPEED]->SetSprite();
}

void BirdItem::CreatePassItem(){
  passItem[0].xpm=(char **)passitem_xpm;
  passItem[1].xpm=(char **)passitem_xpm;
  passItem[2].xpm=(char **)passitem_xpm;
  passItem[3].xpm=(char **)passitem_xpm;
  passItem[4].xpm=(char **)passitem_xpm;
  passItem[5].xpm=(char **)passitem_xpm;
  passItem[6].xpm=(char **)passitem_xpm;
  passItem[7].xpm=(char **)passitem_xpm;
  itemBox[ITEM_PASS]= new SpriteBox(passItem,1,8);
  itemBox[ITEM_PASS]->SetSprite();
}
