#ifndef BIRD_KEYCONFIG_H_
#define BIRD_KEYCONFIG_H_

#include "basemap.h"
#include "mainsdl.h"

#define KEYCONFIG_STATE_TOP 0
#define KEYCONFIG_STATE_KEYBOARD 1
#define KEYCONFIG_STATE_JOYSTICK 2
#define KEYCONFIG_STATE_NUM 3

#define KEYCONFIG_TOP_PX1 400
#define KEYCONFIG_TOP_PY1 500
#define KEYCONFIG_TOP_MX1 0
#define KEYCONFIG_TOP_MY1 60
#define KEYCONFIG_TOP_SX1 0
#define KEYCONFIG_TOP_SY1 0
#define KEYCONFIG_TOP_LF1 FONT_LINE_LEFT
#define KEYCONFIG_TOP_SF1 14
#define KEYCONFIG_TOP_TF1 FONT_SERIF
#define KEYCONFIG_TOP_FF1 FONT_FACE_BOLD

#define KEYCONFIG_TOP_KEYBOARD 0
#define KEYCONFIG_TOP_JOYSTICK 1
#define KEYCONFIG_TOP_NUM 2

#define KEYCONFIG_KEYBOARD_PX1 120
#define KEYCONFIG_KEYBOARD_PY1 340
#define KEYCONFIG_KEYBOARD_MX1 0
#define KEYCONFIG_KEYBOARD_MY1 90
#define KEYCONFIG_KEYBOARD_SX1 0
#define KEYCONFIG_KEYBOARD_SY1 0
#define KEYCONFIG_KEYBOARD_LF1 FONT_LINE_LEFT
#define KEYCONFIG_KEYBOARD_SF1 14
#define KEYCONFIG_KEYBOARD_TF1 FONT_SERIF
#define KEYCONFIG_KEYBOARD_FF1 FONT_FACE_BOLD

#define KEYCONFIG_KEYBOARD_PX2 420
#define KEYCONFIG_KEYBOARD_PY2 KEYCONFIG_KEYBOARD_PY1
#define KEYCONFIG_KEYBOARD_MX2 0
#define KEYCONFIG_KEYBOARD_MY2 KEYCONFIG_KEYBOARD_MY1
#define KEYCONFIG_KEYBOARD_SX2 0
#define KEYCONFIG_KEYBOARD_SY2 0
#define KEYCONFIG_KEYBOARD_LF2 FONT_LINE_LEFT
#define KEYCONFIG_KEYBOARD_SF2 KEYCONFIG_KEYBOARD_SF1
#define KEYCONFIG_KEYBOARD_TF2 FONT_SERIF
#define KEYCONFIG_KEYBOARD_FF2 FONT_FACE_BOLD

#define KEYCONFIG_KEYBOARD_OK 0
#define KEYCONFIG_KEYBOARD_CANCEL 1
#define KEYCONFIG_KEYBOARD_LEFT 2
#define KEYCONFIG_KEYBOARD_RIGHT 3
#define KEYCONFIG_KEYBOARD_UP 4
#define KEYCONFIG_KEYBOARD_DOWN 5
#define KEYCONFIG_KEYBOARD_NUM 6

#define KEYCONFIG_JOYSTICK_PX1 120
#define KEYCONFIG_JOYSTICK_PY1 340
#define KEYCONFIG_JOYSTICK_MX1 0
#define KEYCONFIG_JOYSTICK_MY1 90
#define KEYCONFIG_JOYSTICK_SX1 0
#define KEYCONFIG_JOYSTICK_SY1 0
#define KEYCONFIG_JOYSTICK_LF1 FONT_LINE_LEFT
#define KEYCONFIG_JOYSTICK_SF1 14
#define KEYCONFIG_JOYSTICK_TF1 FONT_SERIF
#define KEYCONFIG_JOYSTICK_FF1 FONT_FACE_BOLD

#define KEYCONFIG_JOYSTICK_PX2 420
#define KEYCONFIG_JOYSTICK_PY2 KEYCONFIG_JOYSTICK_PY1
#define KEYCONFIG_JOYSTICK_MX2 0
#define KEYCONFIG_JOYSTICK_MY2 KEYCONFIG_JOYSTICK_MY1
#define KEYCONFIG_JOYSTICK_SX2 0
#define KEYCONFIG_JOYSTICK_SY2 0
#define KEYCONFIG_JOYSTICK_LF2 FONT_LINE_LEFT
#define KEYCONFIG_JOYSTICK_SF2 KEYCONFIG_JOYSTICK_SF1
#define KEYCONFIG_JOYSTICK_TF2 FONT_SERIF
#define KEYCONFIG_JOYSTICK_FF2 FONT_FACE_BOLD

#define KEYCONFIG_JOYSTICK_NAME 0
#define KEYCONFIG_JOYSTICK_HAXIS 1
#define KEYCONFIG_JOYSTICK_VAXIS 2
#define KEYCONFIG_JOYSTICK_OK 3
#define KEYCONFIG_JOYSTICK_CANCEL 4
#define KEYCONFIG_JOYSTICK_NUM 5

#define KEYCONFIG_VY_STATE_TOP 0
#define KEYCONFIG_VY_STATE_JOYSTICK 1
#define KEYCONFIG_VY_STATE_KEYBOARD 2

#define KEYCONFIG_VY_JOYSTICK_NAME 3

class KeyConfig : public BaseMap{
 private:
  int kbShift;
  int jsShift;
  MainSDL *mainSDL;
  Glib::RefPtr<Gdk::GC> blackGC;
  Glib::RefPtr<Gdk::GC> whiteGC;
 public:
  KeyConfig();
  ~KeyConfig();
  void Create(MainSDL *mainSDL0);
  
  void KeyBoardSelect();
  void JoyStickSelect();
  static void(KeyConfig::*KEYCONFIG_TOP_SELECT[KEYCONFIG_TOP_NUM])();

  void TopMenu();

  void NameSelect();
  void HAxisSelect();
  void VAxisSelect();
  void OkSelect();
  void CancelSelect();
  static void(KeyConfig::*KEYCONFIG_JOYSTICK_SELECT
	      [KEYCONFIG_JOYSTICK_NUM])();

  void JoyStickNameShift();
  void JoyStickHAxisShift();
  void JoyStickVAxisShift();
  void JoyStickOkShift();
  void JoyStickCancelShift();
  static void(KeyConfig::*KEYCONFIG_JOYSTICK_SHIFT
	      [KEYCONFIG_JOYSTICK_NUM])();

  void JoyStickMenuShift();
  void JoyStickMenu();

  void KBOkSelect();
  void KBCancelSelect();
  void KBLeftSelect();
  void KBRightSelect();
  void KBUpSelect();
  void KBDownSelect();
  static void(KeyConfig::*KEYCONFIG_KEYBOARD_SELECT
	      [KEYCONFIG_KEYBOARD_NUM])();

  void KBOkShift();
  void KBCancelShift();
  void KBLeftShift();
  void KBRightShift();
  void KBUpShift();
  void KBDownShift();
  static void(KeyConfig::*KEYCONFIG_KEYBOARD_SHIFT
	      [KEYCONFIG_KEYBOARD_NUM])();

  void KeyBoardMenuShift();
  void KeyBoardMenu();

  static void(KeyConfig::*KEYCONFIG_STATE_SELECT[KEYCONFIG_STATE_NUM])();
  
  void ViewMap();
};

#endif
