#include "player.h"
#include "xpm/craft.h"

BirdPlayer::BirdPlayer(){
  
  fireCount=0;

  px=WIDTH/3;
  py=HEIGHT/2;
  width=2;
  height=2;
  mx=PLAYER_MOV_X;
  my=PLAYER_MOV_Y;
  life=0;
  style=PLAYER_NORMAL_CRAFT;
  move=0;
  form=PLAYER_FORM_NORMAL;
  fire=PF_SMALL;
  item=PLAYER_MODE_NORMAL;
  tcount=0;
  dcount=0;
  direct=0;
  seq=0;
  tmp=0;

  target=NULL;
  next=NULL;
  prev=NULL;

  formName[PLAYER_FORM_NORMAL]="NORMAL";
  formName[PLAYER_FORM_CROSS]="CROSS";
  formName[PLAYER_FORM_XREV]="XREV";
  formName[PLAYER_FORM_HOM]="HOMING";
  formName[PLAYER_FORM_WAVE]="WAVE";
  formName[PLAYER_FORM_DIVIDE]="DIVIDE";
  formName[PLAYER_FORM_DROP]="DROP";

  modeLimit[PLAYER_MODE_NORMAL]=0;
  modeName[PLAYER_MODE_NORMAL]="NORMAL";

  modeLimit[PLAYER_MODE_BOOST]=100;
  modeName[PLAYER_MODE_BOOST]="BOOST";

  modeLimit[PLAYER_MODE_SPEED]=400;
  modeName[PLAYER_MODE_SPEED]="SPEED";

  modeLimit[PLAYER_MODE_PASS]=300;
  modeName[PLAYER_MODE_PASS]="PASS";

  CreateNormalCraft();
  CreateYellowCraft();
  CreateGreenCraft();
  CreateBlueCraft();
}

BirdPlayer::~BirdPlayer(){
  delete playerBox[PLAYER_NORMAL_CRAFT];
  delete playerBox[PLAYER_YELLOW_CRAFT];
  delete playerBox[PLAYER_GREEN_CRAFT];
  delete playerBox[PLAYER_BLUE_CRAFT];
}

void BirdPlayer::Create(Glib::RefPtr<Gdk::Pixmap> frame0,
			Glib::RefPtr<Gdk::GC> mGC0,
			PlayerFire *pfireList0,
			BirdLand *landList0,
			int *selectValue0,
			int *okValue0,
			int *cancelValue0){
  frame=frame0;
  mGC=mGC0;
  pfireList=pfireList0;
  landList=landList0;
  selectValue=selectValue0;
  okValue=okValue0;
  cancelValue=cancelValue0;
}

std::string BirdPlayer::FormName(int type0){
  std::string name0;
  
  if(type0<PLAYER_FORM_NUM){
    name0=formName[type0];
  }
  else{
    name0="ERROR";
  }
  return name0;
}

void BirdPlayer::SetMode(int mode0){
  tcount=modeLimit[mode0];
}

void BirdPlayer::InitPlayer(int life0){
  px=WIDTH*1/4;
  py=HEIGHT/2;
  mx=PLAYER_MOV_X;
  my=PLAYER_MOV_Y;
  life=life0;
  style=PLAYER_NORMAL_CRAFT;
  form=PLAYER_FORM_NORMAL;
  fire=PF_SMALL;
  item=PLAYER_MODE_NORMAL;
  move=0;
  tcount=0;
  dcount=0;
  direct=0;
  seq=0;
  tmp=0;
  tx=0;
  ty=0;
  score=0;
  target=NULL;
  next=NULL;
  prev=NULL;
}

void BirdPlayer::InitPlayer(int life0,int fire0,int form0){
  px=WIDTH*1/4;
  py=HEIGHT/2;
  mx=PLAYER_MOV_X;
  my=PLAYER_MOV_Y;
  life=life0;
  style=PLAYER_NORMAL_CRAFT;
  form=form0;
  fire=fire0;
  item=PLAYER_MODE_NORMAL;
  move=0;
  tcount=0;
  dcount=0;
  direct=0;
  seq=0;
  tmp=0;
  tx=0;
  ty=0;
  score=0;
  target=NULL;
  next=NULL;
  prev=NULL;
}

void BirdPlayer::LostItem(){
  if(rand()%3==0){
    fire=PF_SMALL;
  }
  if(rand()%5==0){
    form=PLAYER_FORM_NORMAL;
  }
}

void BirdPlayer::DrawSprite(int style,int direct,int px,int py){
  SpriteBox *spriteBox;
  
  spriteBox=playerBox[style];
  
  spriteBox->DrawSprite(frame,mGC,px,py,direct);
}

int BirdPlayer::ContactLand(int *tX,int *tY){
  BirdRhoum *node;
  BirdRhoum *tmp;
  int flag0=0;
  int pX,pY;

  node=landList->first;
  while(node){
    tmp=node->next;
    
    pX=px;
    pY=py;
    flag0=landList->ContactPoint(node,&pX,&pY);
    
    if(flag0!=0){
      *tX=pX;
      *tY=pY;
      break;
    }
    node=tmp;
  }
  return flag0;
}

void BirdPlayer::SelectAction0(){
  //DUMMY
}

void BirdPlayer::SelectAction1(){
  px-=mx;
}
void BirdPlayer::SelectAction2(){
  py-=my;
}
void BirdPlayer::SelectAction3(){
  px-=mx;
  py-=my;
}
void BirdPlayer::SelectAction4(){
  px+=mx;
}
void BirdPlayer::SelectAction5(){
  //DUMMY
}
void BirdPlayer::SelectAction6(){
  px+=mx;
  py-=my;
}
void BirdPlayer::SelectAction7(){
  px-=mx;
}
void BirdPlayer::SelectAction8(){
  py+=my;
}
void BirdPlayer::SelectAction9(){
  px-=mx;
  py+=my;
}
void BirdPlayer::SelectAction10(){
  //DUMMY
}
void BirdPlayer::SelectAction11(){
  px-=mx;
}
void BirdPlayer::SelectAction12(){
  px+=mx;
  py+=my;
}
void BirdPlayer::SelectAction13(){
  py+=my;
}
void BirdPlayer::SelectAction14(){
  px+=mx;
}
void BirdPlayer::SelectAction15(){
  //DUMMY
}
void (BirdPlayer::*BirdPlayer::SELECT_ACTION[16])()={
  &BirdPlayer::SelectAction0,
  &BirdPlayer::SelectAction1,
  &BirdPlayer::SelectAction2,
  &BirdPlayer::SelectAction3,
  &BirdPlayer::SelectAction4,
  &BirdPlayer::SelectAction5,
  &BirdPlayer::SelectAction6,
  &BirdPlayer::SelectAction7,
  &BirdPlayer::SelectAction8,
  &BirdPlayer::SelectAction9,
  &BirdPlayer::SelectAction10,
  &BirdPlayer::SelectAction11,
  &BirdPlayer::SelectAction12,
  &BirdPlayer::SelectAction13,
  &BirdPlayer::SelectAction14,
  &BirdPlayer::SelectAction15
};

void BirdPlayer::Animate(){
    SpriteBox *spriteBox0;
  //  spriteBox0=playerBox[0];
  int landFlag=0,tX=0,tY=0;
  int i;

  if(tcount>0){
    if(item==PLAYER_MODE_SPEED){
      style=PLAYER_GREEN_CRAFT;
      mx=PLAYER_MOV_X*2;
      my=PLAYER_MOV_Y*2;
    }
    else if(item==PLAYER_MODE_BOOST){
      style=PLAYER_YELLOW_CRAFT;
      mx=PLAYER_MOV_X;
      my=PLAYER_MOV_Y;
    }
    else if(item==PLAYER_MODE_PASS){
      style=PLAYER_BLUE_CRAFT;
      mx=PLAYER_MOV_X;
      my=PLAYER_MOV_Y;
    }
    else{
      mx=PLAYER_MOV_X;
      my=PLAYER_MOV_Y;
    }
      tcount--;
  }
  else{
    if(item==PLAYER_MODE_BOOST){
      item=PLAYER_MODE_NORMAL;
    }
    else if(item==PLAYER_MODE_SPEED){
      item=PLAYER_MODE_NORMAL;
      mx=PLAYER_MOV_X;
      my=PLAYER_MOV_Y;
    }
    else if(item==PLAYER_MODE_PASS){
      item=PLAYER_MODE_NORMAL;
    }
    style=PLAYER_NORMAL_CRAFT;
  }
  
  (this->*SELECT_ACTION[*selectValue])();
  
  if(target!=NULL){
    if(target->life>0){
      
    }
    else{
      target=NULL;
    }
  }

  if(*okValue){
    
    if(item==PLAYER_MODE_BOOST){
      fireCount=-1; 
    }

    if(fireCount>5){
      fireCount=-1;
    }
    else if(fireCount>0){
      
    }
    else{
      (this->*PLAYER_FORM[form])();
    }
    fireCount++;
  }
  else{
    if(fireCount>2){
      fireCount=0;
    }
  }
    
    
  if(px>WIDTH){
    px=WIDTH-1;
  }
  else if(px<0){
    px=1;
  }
  if(py>HEIGHT){
    py=HEIGHT-1;
  }
  else if(py<0){
    py=1;
  }

  if(item==PLAYER_MODE_PASS){
    landFlag=0;
  }
  else{
    landFlag=ContactLand(&tX,&tY);
  }
  
  if(landFlag){
    
    if(dcount>0){
      
    }
    else{
      //    life--;
      dcount=PLAYER_T_DMG;
    }
    
    if(landFlag==1){
      px=tX+PLAYER_MOV_X;
      py=tY+PLAYER_MOV_Y;
    }
    else if(landFlag==2){
      px=tX-PLAYER_MOV_X;
      py=tY+PLAYER_MOV_Y;
    }
    else if(landFlag==3){
      px=tX+PLAYER_MOV_X;
      py=tY-PLAYER_MOV_Y;
    }
    else if(landFlag==4){
      px=tX-PLAYER_MOV_X;
      py=tY-PLAYER_MOV_Y;
    }
  }

  if(px>WIDTH){
    px=WIDTH-1;
  }
  else if(px<0){
    px=1;
  }
  if(py>HEIGHT){
    py=HEIGHT-1;
  }
  else if(py<0){
    py=1;
  }
  
  spriteBox0=playerBox[style];
  
  seq++;
  if(seq>=spriteBox0->nSeq){
    seq=0;
  }

  if(dcount==PLAYER_T_DMG){
    life--;
    LostItem();
  }
  else{
    
  }
  
  if(dcount>0){

    dcount--;
    spriteBox0->DrawSprite
      (frame,mGC,px,py,
       seq+direct*spriteBox0->nSeq+spriteBox0->nLength/2);
  }
  else{
    spriteBox0->DrawSprite
      (frame,mGC,px,py,
       seq+direct*spriteBox0->nSeq);
  }
}

void BirdPlayer::NormalForm(){
  
  pfireList->Appear(px,py,
		    pfireList->PF_WIDTH[fire],
		    pfireList->PF_HEIGHT[fire],
		    10,0,pfireList->PF_LIFE[fire],
		    fire,PF_MOV_STR,PF_TOUCH_PAIN,
		    PF_FORM_LR,0);
  
}
void BirdPlayer::CrossForm(){
      pfireList->Appear(px,py,
			pfireList->PF_WIDTH[fire],
			pfireList->PF_HEIGHT[fire],
			10,0,pfireList->PF_LIFE[fire],
			fire,PF_MOV_STR,PF_TOUCH_PAIN,
			PF_FORM_LR,0);
      pfireList->Appear(px,py,
			pfireList->PF_WIDTH[fire],
			pfireList->PF_HEIGHT[fire],
			-10,0,pfireList->PF_LIFE[fire],
			fire,PF_MOV_STR,PF_TOUCH_PAIN,
			PF_FORM_LR,0);

      pfireList->Appear(px,py,
			pfireList->PF_HEIGHT[fire],
			pfireList->PF_WIDTH[fire],
			0,10,pfireList->PF_LIFE[fire],
			fire,PF_MOV_STR,PF_TOUCH_PAIN,
			PF_FORM_UD,0);
      pfireList->Appear(px,py,
			pfireList->PF_HEIGHT[fire],
			pfireList->PF_WIDTH[fire],
			0,-10,pfireList->PF_LIFE[fire],
			fire,PF_MOV_STR,PF_TOUCH_PAIN,
			PF_FORM_UD,0);
}
void BirdPlayer::XForm(){
  pfireList->Appear(px,py,
		    pfireList->PF_WIDTH[fire],
		    pfireList->PF_HEIGHT[fire],
		    8,-8,pfireList->PF_LIFE[fire],
		    fire,PF_MOV_REV,PF_TOUCH_REV,
		    PF_FORM_LR,6);
  pfireList->Appear(px,py,
		    pfireList->PF_WIDTH[fire],
		    pfireList->PF_HEIGHT[fire],
		    8,8,pfireList->PF_LIFE[fire],
		    fire,PF_MOV_REV,PF_TOUCH_REV,
		    PF_FORM_LR,6);

  pfireList->Appear(px,py,
		    pfireList->PF_WIDTH[fire],
		    pfireList->PF_HEIGHT[fire],
		    -8,-8,pfireList->PF_LIFE[fire],
		    fire,PF_MOV_REV,PF_TOUCH_REV,
		    PF_FORM_LR,6);
  pfireList->Appear(px,py,
		    pfireList->PF_WIDTH[fire],
		    pfireList->PF_HEIGHT[fire],
		    -8,8,pfireList->PF_LIFE[fire],
		    fire,PF_MOV_REV,PF_TOUCH_REV,
		    PF_FORM_LR,6);
}
void BirdPlayer::WaveForm(){
  pfireList->Appear(px,py,0,-9,10,2,
		PF_MOV_HOM_ACCE,PF_TOUCH_HOM,
		PF_FORM_LR,PF_TARGET_FIXED);
  pfireList->Appear(px,py,0,9,10,2,
		PF_MOV_HOM_ACCE,PF_TOUCH_HOM,
		PF_FORM_LR,PF_TARGET_FIXED);
  pfireList->Appear(px,py,0,0,10,2,
		PF_MOV_HOM_ACCE,PF_TOUCH_HOM,
		PF_FORM_LR,PF_TARGET_VARIABLE);
}
void BirdPlayer::HomingForm(){
  pfireList->Appear(px+8,py,10,0,10,0,
		PF_MOV_HOM,PF_TOUCH_HOM,
		PF_FORM_LR,PF_TARGET_VARIABLE);
  pfireList->Appear(px-8,py,-10,0,10,0,
		PF_MOV_HOM,PF_TOUCH_HOM,
		PF_FORM_LR,PF_TARGET_VARIABLE);
}
void BirdPlayer::DivideForm(){
  pfireList->Appear(px,py,10,0,0,0,
		    PF_MOV_DISPARSE,PF_TOUCH_DISPARSE,
		    PF_FORM_LR,0);
}
void BirdPlayer::DropForm(){
  pfireList->Appear(px,py,10,0,5,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
  pfireList->Appear(px,py,-10,0,5,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
  /*
  pfireList->Appear(px,py,8,-6,10,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
  pfireList->Appear(px,py,-8,-6,10,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
  */
  pfireList->Appear(px,py,6,0,14,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
  pfireList->Appear(px,py,-6,0,14,1,
		    PF_MOV_GROUND,PF_TOUCH_GROUND,
		    PF_FORM_LR,0);
}
void (BirdPlayer::*BirdPlayer::PLAYER_FORM[PLAYER_FORM_NUM])()={
  &BirdPlayer::NormalForm,
  &BirdPlayer::CrossForm,
  &BirdPlayer::XForm,
  &BirdPlayer::WaveForm,
  &BirdPlayer::HomingForm,
  &BirdPlayer::DivideForm,
  &BirdPlayer::DropForm,
};


void BirdPlayer::CreateNormalCraft(){
  normalcraft[0].xpm=(char **)craft_f1_xpm;
  normalcraft[1].xpm=(char **)craft_f1_xpm;
  normalcraft[2].xpm=(char **)craft_b1_xpm;
  normalcraft[3].xpm=(char **)craft_b1_xpm;
  normalcraft[4].xpm=(char **)craft_f1_xpm;
  normalcraft[5].xpm=(char **)craft_fE1_xpm;
  normalcraft[6].xpm=(char **)craft_b1_xpm;
  normalcraft[7].xpm=(char **)craft_bE1_xpm;

  playerBox[PLAYER_NORMAL_CRAFT]= new SpriteBox(normalcraft,2,8); 
  playerBox[PLAYER_NORMAL_CRAFT]->SetSprite();
}

void BirdPlayer::CreateYellowCraft(){
  yellowcraft[0].xpm=(char **)yellowcraft_f1_xpm;
  yellowcraft[1].xpm=(char **)yellowcraft_f1_xpm;
  yellowcraft[2].xpm=(char **)yellowcraft_b1_xpm;
  yellowcraft[3].xpm=(char **)yellowcraft_b1_xpm;
  yellowcraft[4].xpm=(char **)craft_fE1_xpm;
  yellowcraft[5].xpm=(char **)yellowcraft_f1_xpm;
  yellowcraft[6].xpm=(char **)craft_bE1_xpm;
  yellowcraft[7].xpm=(char **)yellowcraft_b1_xpm;

  playerBox[PLAYER_YELLOW_CRAFT]= new SpriteBox(yellowcraft,2,8); 
  playerBox[PLAYER_YELLOW_CRAFT]->SetSprite();
}

void BirdPlayer::CreateGreenCraft(){
  greencraft[0].xpm=(char **)greencraft_f1_xpm;
  greencraft[1].xpm=(char **)greencraft_f1_xpm;
  greencraft[2].xpm=(char **)greencraft_b1_xpm;
  greencraft[3].xpm=(char **)greencraft_b1_xpm;
  greencraft[4].xpm=(char **)craft_fE1_xpm;
  greencraft[5].xpm=(char **)greencraft_f1_xpm;
  greencraft[6].xpm=(char **)craft_bE1_xpm;
  greencraft[7].xpm=(char **)greencraft_b1_xpm;

  playerBox[PLAYER_GREEN_CRAFT]= new SpriteBox(greencraft,2,8); 
  playerBox[PLAYER_GREEN_CRAFT]->SetSprite();
}

void BirdPlayer::CreateBlueCraft(){
  bluecraft[0].xpm=(char **)bluecraft_f1_xpm;
  bluecraft[1].xpm=(char **)bluecraft_f1_xpm;
  bluecraft[2].xpm=(char **)bluecraft_b1_xpm;
  bluecraft[3].xpm=(char **)bluecraft_b1_xpm;
  bluecraft[4].xpm=(char **)craft_fE1_xpm;
  bluecraft[5].xpm=(char **)bluecraft_f1_xpm;
  bluecraft[6].xpm=(char **)craft_bE1_xpm;
  bluecraft[7].xpm=(char **)bluecraft_b1_xpm;

  playerBox[PLAYER_BLUE_CRAFT]= new SpriteBox(bluecraft,2,8); 
  playerBox[PLAYER_BLUE_CRAFT]->SetSprite();
}
