#ifndef BIRD_ENEMY_H_
#define BIRD_ENEMY_H_

#include <gtkmm.h>

#include "actlist.h"
#include "sprite.h"
#include "land.h"
#include "player.h"
#include "playerfire.h"
#include "enemyfire.h"
#include "item.h"
#include "map.h"

#define EN_STR_BIRD 0
#define EN_REV_BIRD 1
#define EN_HOM_BIRD 2
#define EN_SMALL_SPHENICS 3
#define EN_WIDE_SPHENICS 4
#define EN_NORMAL_STRUTHIO 5
#define EN_NORMAL_GALLUS 6
#define EN_SMALL_BAT 7
#define EN_BIG_BAT 8
#define EN_NORMAL_EAGLE 9
#define EN_LINE_EAGLE 10
#define EN_SMALL_CROSSSTRIX 11
#define EN_SMALL_XREVSTRIX 12
#define EN_CROW 13
#define EN_STYLE_NUM 14

#define EN_WAY_F 0
#define EN_WAY_B 1
#define EN_WAY_NUM 2

#define EN_MOV_STR 0
#define EN_MOV_REV 1
#define EN_MOV_HOM 2
#define EN_MOV_NOSTR 3
#define EN_MOV_GROUND 4
#define EN_MOV_JUMP 5
#define EN_MOV_HOMSTOP 6
#define EN_MOV_BOSS 7
#define EN_MOV_XYACCE 8
#define EN_MOV_ORDER 9
#define EN_MOV_ALLSTR 10
#define EN_MOV_ALLREV 11
#define EN_MOV_NUM 12

#define EN_TOUCH_PAIN 0
#define EN_TOUCH_REV 1
#define EN_TOUCH_CH_NOSTR 2
#define EN_TOUCH_BLOW 3
#define EN_TOUCH_JUMP 4
#define EN_TOUCH_THROUGH 5
#define EN_TOUCH_NUM 6

#define EN_FORM_NOFIRE 0
#define EN_FORM_STR_SMALL 1
#define EN_FORM_STR_BIG 2
#define EN_FORM_STR_WIDE 3
#define EN_FORM_STR_LINE 4
#define EN_FORM_TAR_SMALL 5
#define EN_FORM_TAR_BIG 6
#define EN_FORM_TAR_WIDE 7
#define EN_FORM_TAR_LINE 8
#define EN_FORM_HOM_SMALL 9
#define EN_FORM_HOM_BIG 10
#define EN_FORM_HOM_WIDE 11
#define EN_FORM_HOM_LINE 12
#define EN_FORM_CROSS_SMALL 13
#define EN_FORM_XREV_SMALL 14
#define EN_FORM_ORDER 15
#define EN_FORM_NUM 16

#define EN_FORM_FREQ 1000

#define EN_ITEMDROP_WEAPON 0
#define EN_ITEMDROP_ALL 1
#define EN_ITEMDROP_WEAPONBONUS 2
#define EN_ITEMDROP_BONUSLIFE 3
#define EN_ITEMDROP_NUM 4

#define ENEMY_T_DMG 5
#define ENEMY_T_DEAD 10

class BirdPlayer;
class PlayerFire;
class EnemyFire;
class BirdLand;
class BirdItem;

class BirdEnemy : public ActorList {
 public:
  Glib::RefPtr<Gdk::Pixmap> frame;
  Glib::RefPtr<Gdk::GC> mGC;
  BirdSprite bird[12];
  BirdSprite revbird[12];
  BirdSprite hombird[12];
  BirdSprite crow[12];
  BirdSprite smallspenics[4];
  BirdSprite widespenics[4];
  BirdSprite struthio[8];
  BirdSprite gallus[4];
  BirdSprite smallbat[8];
  BirdSprite bigbat[8];
  BirdSprite normaleagle[4];
  BirdSprite lineeagle[4];
  BirdSprite smallcrossstrix[8];
  BirdSprite smallxrevstrix[8];
  SpriteBox *enemyBox[20];
  BirdLand *landList;
  BirdPlayer *player;
  PlayerFire *pfireList;
  EnemyFire *efireList;
  BirdItem *itemList;
  BirdMap *Map;
  BirdEnemy();
  ~BirdEnemy();
  void Create(Glib::RefPtr<Gdk::Pixmap> frame0,
	      Glib::RefPtr<Gdk::GC> mGC0,
	      BirdMap *Map0,
	      BirdPlayer *player0,
	      BirdLand *landList0,
	      PlayerFire *pfireList0,
	      EnemyFire *efireList0,
	      BirdItem *itemList0);

  void DrawSprite(int style,int direct,int seq,int px,int py);
  void DrawScale
    (int style,int direct,int seq,int px,int py,int width,int height);

  int ContactPlayerFire(BirdActor *act);
  int ContactLand(BirdActor *act,int *tX,int *tY);

  BirdActor *Appear(int px,int py,int width,int height,
		    int mx,int my,int tx,int ty,int life,int score,
		    int style,int move,int touch,int tcount,
		    int form,int fire,int item,int drop);

  int Animate();
  void StraightMove(BirdActor *act,int speedX,int speedY);
  void ReverseMove(BirdActor *act,int speedX,int speedY);
  void HomingMove(BirdActor *act,int speedX,int speedY);
  void HomingStopMove(BirdActor *act,int speedX,int speedY);
  void NoStrMove(BirdActor *act,int speedX,int speedY);
  void GroundMove(BirdActor *act,int speedX,int speedY);
  void JumpMove(BirdActor *act,int speedX,int speedY);
  void BossMove(BirdActor *act,int speedX,int speedY);
  void XYAccelerateMove(BirdActor *act,int speedX,int speedY);
  void OrderMove(BirdActor *act,int speedX,int speedY);
  void ALLStrMove(BirdActor *act,int speedX,int speedY);
  void ALLRevMove(BirdActor *act,int speedX,int speedY);

  static void(BirdEnemy::*ENEMY_MOVE[EN_MOV_NUM])
    (BirdActor *act,int speedX,int speedY);

  void ThroughTouch(BirdActor *act,int playerFlag);
  void PainTouch(BirdActor *act,int playerFlag);
  void BlowTouch(BirdActor *act,int playerFlag);
  void ReverseTouch(BirdActor *act,int playerFlag);
  void ChangeNoStrTouch(BirdActor *act,int playerFlag);
  void JumpTouch(BirdActor *act,int playerFlag);

  static void(BirdEnemy::*ENEMY_TOUCH[EN_TOUCH_NUM])
    (BirdActor *act,int playerFlag);

  void NoFireForm(BirdActor *act);
  void SmallStrFireForm(BirdActor *act);
  void BigStrFireForm(BirdActor *act);
  void WideStrFireForm(BirdActor *act);
  void LineStrFireForm(BirdActor *act);
  void SmallTargetFireForm(BirdActor *act);
  void BigTargetFireForm(BirdActor *act);
  void WideTargetFireForm(BirdActor *act);
  void LineTargetFireForm(BirdActor *act);
  void SmallHomingFireForm(BirdActor *act);
  void BigHomingFireForm(BirdActor *act);
  void WideHomingFireForm(BirdActor *act);
  void LineHomingFireForm(BirdActor *act);
  void SmallCrossFireForm(BirdActor *act);
  void SmallXRevFireForm(BirdActor *act);
  void OrderFireForm(BirdActor *act);
  static void(BirdEnemy::*ENEMY_FIRE_FORM[EN_FORM_NUM])(BirdActor *act);

  void WeaponItemDrop(BirdActor *act);
  void ALLItemDrop(BirdActor *act);
  void WeaponBonusItemDrop(BirdActor *act);
  void BonusLifeItemDrop(BirdActor *act);
  static void(BirdEnemy::*ENEMY_ITEMDROP_SELECT[EN_ITEMDROP_NUM])
    (BirdActor *act);
  void ItemDropSelect(BirdActor *act);

  void CreateNormalBird();
  void CreateReverseBird();
  void CreateHomingBird();

  void CreateSmallSphenics();
  void CreateWideSphenics();

  void CreateNormalStruthio();

  void CreateNormalGallus();

  void CreateSmallBat();
  void CreateBigBat();

  void CreateNormalEagle();
  void CreateLineEagle();

  void CreateSmallCrossStrix();
  void CreateSmallXRevStrix();

  void CreateCrow();
};

#endif
