#ifndef BIRD_PLAYER_H_
#define BIRD_PLAYER_H_

#include <gtkmm.h>

#include "actlist.h"
#include "sprite.h"
#include "playerfire.h"
#include "land.h"

#define PLAYER_T_DMG 40

#define PLAYER_NORMAL_CRAFT 0
#define PLAYER_YELLOW_CRAFT 1
#define PLAYER_GREEN_CRAFT 2
#define PLAYER_BLUE_CRAFT 3
#define PLAYER_TYPE_NUM 4

#define PLAYER_MOV_X 8
#define PLAYER_MOV_Y 8

#define PLAYER_TARGET_VARIABLE 0
#define PLAYER_TARGET_FIXED 1
#define PLAYER_TARGET_NUM 2

#define PLAYER_FORM_NORMAL 0
#define PLAYER_FORM_CROSS 1
#define PLAYER_FORM_XREV 2
#define PLAYER_FORM_WAVE 3
#define PLAYER_FORM_HOM 4
#define PLAYER_FORM_DIVIDE 5
#define PLAYER_FORM_DROP 6
#define PLAYER_FORM_NUM 7

#define PLAYER_MODE_NORMAL 0
#define PLAYER_MODE_BOOST 1
#define PLAYER_MODE_SPEED 2
#define PLAYER_MODE_PASS 3
#define PLAYER_MODE_NUM 4

class PlayerFire;

class BirdLand;

class BirdPlayer : public BirdActor{
 private:
  Glib::RefPtr<Gdk::Pixmap> frame;
  Glib::RefPtr<Gdk::GC> mGC;
  PlayerFire *pfireList;
  BirdLand *landList;
  BirdSprite normalcraft[8];
  BirdSprite yellowcraft[8];
  BirdSprite greencraft[8];
  BirdSprite bluecraft[8];
  SpriteBox *playerBox[PLAYER_TYPE_NUM];
  int *selectValue;
  int *okValue;
  int *cancelValue;
  int fireCount;
 public:
  std::string formName[PLAYER_FORM_NUM];
  std::string modeName[PLAYER_MODE_NUM];
  int modeLimit[PLAYER_MODE_NUM];
  BirdPlayer();
  ~BirdPlayer();
  void Create(Glib::RefPtr<Gdk::Pixmap> frame0,
	      Glib::RefPtr<Gdk::GC> mGC0,
	      PlayerFire *pfireList0,
	      BirdLand *landList0,
	      int *selectValue0,int *okValue0,int *cancelValue0);

  std::string FormName(int type0);

  void SetMode(int mode0);
  void InitPlayer(int life0);
  void InitPlayer(int life0,int fire0,int form0);
  void LostItem();

  void DrawSprite(int style,int direct,int px,int py);

  int ContactLand(int *tX,int *tY);

  void SelectAction0();
  void SelectAction1();
  void SelectAction2();
  void SelectAction3();
  void SelectAction4();
  void SelectAction5();
  void SelectAction6();
  void SelectAction7();
  void SelectAction8();
  void SelectAction9();
  void SelectAction10();
  void SelectAction11();
  void SelectAction12();
  void SelectAction13();
  void SelectAction14();
  void SelectAction15();
  static void(BirdPlayer::*SELECT_ACTION[16])();

  void Animate();

  void NormalForm();
  void CrossForm();
  void XForm();
  void WaveForm();
  void HomingForm();
  void DivideForm();
  void DropForm();
  static void(BirdPlayer::*PLAYER_FORM[PLAYER_FORM_NUM])();

  void CreateNormalCraft();
  void CreateYellowCraft();
  void CreateGreenCraft();
  void CreateBlueCraft();
};

#endif
