#ifndef BIRD_PLAYER_FIRE_H_
#define BIRD_PLAYER_FIRE_H_

#include <gtkmm.h>
#include "actlist.h"
#include "sprite.h"
#include "land.h"
#include "enemy.h"
#include "player.h"
#include "item.h"

#define PF_POW_WIDTH 10
#define PF_POW_HEIGHT 10

#define PF_SMALL 0
#define PF_BIG 1
#define PF_WIDE 2
#define PF_LINE 3
#define PF_TYPE_NUM 4

#define PF_MOV_STR 0
#define PF_MOV_REV 1
#define PF_MOV_HOM 2
#define PF_MOV_NOSTR 3
#define PF_MOV_HOM_XACCE 4
#define PF_MOV_HOM_XYACCE 5
#define PF_MOV_HOM_ACCE 6
#define PF_MOV_DISPARSE 7
#define PF_MOV_GROUND 8
#define PF_MOV_NUM 9

#define PF_DIRECT_R 0
#define PF_DIRECT_L 1
#define PF_DIRECT_U 2
#define PF_DIRECT_D 3
#define PF_DIRECT_NUM 4

#define PF_FORM_LR 0
#define PF_FORM_UD 1
#define PF_FORM_NUM 2

#define PF_TOUCH_PAIN 0
#define PF_TOUCH_REV 1
#define PF_TOUCH_HOM 2
#define PF_TOUCH_BLOW 3
#define PF_TOUCH_DISPARSE 4
#define PF_TOUCH_GROUND 5
#define PF_TOUCH_NUM 6

#define PF_TOUCH_FLAG_FALSE 0
#define PF_TOUCH_FLAG_FIRE 1
#define PF_TOUCH_FLAG_ENEMY 2
#define PF_TOUCH_FLAG_NUM 3

#define PF_TARGET_VARIABLE 0
#define PF_TARGET_FIXED 1
#define PF_TARGET_NUM 2

#define PF_ITEMDROP_NORMAL 0
#define PF_ITEMDROP_MAINWEPON 1
#define PF_ITEMDROP_NUM 2

class BirdLand;
class BirdEnemy;
class BirdPlayer;
class BirdItem;

class PlayerFire : public ActorList{
 public:
  Glib::RefPtr<Gdk::Pixmap> frame;
  Glib::RefPtr<Gdk::GC> mGC;
  int powHFire[PF_TYPE_NUM];
  int powVFire[PF_TYPE_NUM];
  int PF_WIDTH[PF_TYPE_NUM];
  int PF_HEIGHT[PF_TYPE_NUM];
  int PF_LIFE[PF_TYPE_NUM];
  std::string fireName[PF_TYPE_NUM];
  BirdSprite smallPF[8];
  BirdSprite bigPF[8];
  BirdSprite widePF[8];
  BirdSprite linePF[8];
  SpriteBox *pfireBox[PF_TYPE_NUM];
  BirdLand *landList;
  BirdEnemy *enemyList;
  BirdItem *itemList;
  BirdPlayer *player;
  PlayerFire();
  ~PlayerFire();
  void Create(Glib::RefPtr<Gdk::Pixmap> frame0,
	      Glib::RefPtr<Gdk::GC> mGC0,
	      BirdPlayer *player0,
	      BirdEnemy *enemyList0,
	      BirdLand *landList0,
	      BirdItem *itemList0);

  std::string FireName(int type0);

  void ContactEnemy(BirdActor *act);
  int ContactLand(BirdActor *act);
  int ContactLand(BirdActor *act,int *tX,int *tY);
  
  int CheckHomingTarget(BirdActor *act);
  void SetHomingVariableTarget
    (BirdActor *act,BirdActor *target,int flag);
  void SetHomingFixedTarget(BirdActor *act,BirdActor *target,int flag);

  void NoSetMove(BirdActor *act,int value);
  void SetHomingMove(BirdActor *act,int value);
  void SetHomUnLock(BirdActor *act,int value);
  static void(PlayerFire::*PF_SETMOVE_SELECT[PF_MOV_NUM])
    (BirdActor *act,int value);

  void Appear(int px,int py,int mx,int my,int tx,int ty,
	      int move,int touch,int form,int value);

  void Appear(int px,int py,int width,int height,
	      int mx,int my,int life,
	      int style,int move,int touch,
	      int form,int tcount);
  int Animate();

  void Disparse8Way(BirdActor *act);

  void StraightMove(BirdActor *act,int landFlag);
  void NoStrMove(BirdActor *act,int landFlag);
  void ReverseMove(BirdActor *act,int landFlag);
  void HomingMove(BirdActor *act,int landFlag);
  void HomXAcceMove(BirdActor *act,int landFlag);
  void HomXYAcceMove(BirdActor *act,int landFlag);
  void AccelerateMove(BirdActor *act,int landFlag);
  void DisparseMove(BirdActor *act,int landFlag);
  void GroundMove(BirdActor *act,int landFlag);
  static void(PlayerFire::*PLAYER_FIRE_MOVE[PF_MOV_NUM])
    (BirdActor *act,int landFlag);

  int PainTouch(BirdActor *fire,BirdActor *enemy,
		int enemyFlag);
  int ReverseTouch(BirdActor *fire,BirdActor *enemy,
		   int enemyFlag);
  int HomingTouch(BirdActor *fire,BirdActor *enemy,
		  int enemyFlag);
  int BlowTouch(BirdActor *fire,BirdActor *enemy,
		int enemyFlag);
  int DisparseTouch(BirdActor *fire,BirdActor *enemy,
		int enemyFlag);
  int GroundTouch(BirdActor *fire,BirdActor *enemy,
		int enemyFlag);
  static int(PlayerFire::*PLAYER_FIRE_TOUCH[PF_TOUCH_NUM])
    (BirdActor *fire,BirdActor *enemy,int enemyFlag);
  int EnemyTouchSelect
    (BirdActor *fire,BirdActor *enemy,int enemyFlag);

  void MainWeponItemDrop(BirdActor *act);
  void NormalItemDrop(BirdActor *act);
  static void(PlayerFire::*PLAYER_FIRE_ITEMDROP[PF_ITEMDROP_NUM])
    (BirdActor *act);

  void CreateSmallPF();
  void CreateBigPF();
  void CreateWidePF();
  void CreateLinePF();
};

#endif
