#ifndef BIRD_SCORE_FILE_H_
#define BIRD_SCORE_FILE_H_

#include <gtkmm.h>
#include <iostream>
#include <fstream>
#include <ctime>

#define SCORE_CALCID_POWMODE 1234

#define SCORE_ELEMENT_SCORE 0
#define SCORE_ELEMENT_LIFE 1
#define SCORE_ELEMENT_SEC 2
#define SCORE_ELEMENT_MODE 3
#define SCORE_ELEMENT_ID 4
#define SCORE_ELEMENT_STAGEID 5
#define SCORE_ELEMENT_TIME 6
#define SCORE_ELEMENT_STAGE 7
#define SCORE_ELEMENT_NAME 8
#define SCORE_ELEMENT_NUM 9

#define SCORE_FORM_ASCEND 0
#define SCORE_FORM_DESCEND 1
#define SCORE_FORM_NUM 2

#define SCORE_MODE_STREAM 0
#define SCORE_MODE_BOSSPRACTICE 1
#define SCORE_MODE_CREATESTAGE 2
#define SCORE_MODE_NUM 3

/*
#define SCORE_MODE_BOSSSILKHAT 2
#define SCORE_MODE_BOSSCROWD 3
#define SCORE_MODE_BOSSEAGLE 4
*/

class BirdScore{
 public:
  int score;
  int life;
  int sec;
  int mode;
  int id;
  int stageID;
  int time;
  std::string stage;
  std::string name;
  BirdScore *next;
  BirdScore *prev;
};

class ScoreFile{
 public:
  std::string file;
  int listSize;
  BirdScore *first;
  BirdScore *end;
  ScoreFile();
  ~ScoreFile();
  void Create();

  int CalcString(const char *str,int length);

  int CalcID(int score0,int sec0,int mode0,
	     std::string stage,std::string name);
  void Append(BirdScore *node);
  void Prepend(BirdScore *node);
  void Insert(BirdScore *node,BirdScore *target);
  void Reject(BirdScore *node);
  void ALLDelete();

  void WriteFile(int score0,int life0,int sec0,int mode0,
		 int stageID0,std::string stage,std::string name);

  void EnterScore(std::string *data,int length);
  void ReadFile();
  void ViewList();

  void SortList(int element,int form);

  int ScoreCompare(BirdScore *node,BirdScore *target);
  int LifeCompare(BirdScore *node,BirdScore *target);
  int SecCompare(BirdScore *node,BirdScore *target);
  int ModeCompare(BirdScore *node,BirdScore *target);
  int IDCompare(BirdScore *node,BirdScore *target);
  int StageIDCompare(BirdScore *node,BirdScore *target);
  int TimeCompare(BirdScore *node,BirdScore *target);
  int StageCompare(BirdScore *node,BirdScore *target);
  int NameCompare(BirdScore *node,BirdScore *target);
  static int(ScoreFile::*SCORE_COMPARE_SELECT[SCORE_ELEMENT_NUM])
    (BirdScore *node,BirdScore *target);
  int CompareSelect
    (BirdScore *node,BirdScore *target,int element);

  int ScoreValue(BirdScore *score0);
  int LifeValue(BirdScore *score0);
  int SecValue(BirdScore *score0);
  int ModeValue(BirdScore *score0);
  int IDValue(BirdScore *score0);
  int StageIDValue(BirdScore *score0);
  int TimeValue(BirdScore *score0);
  int StageValue(BirdScore *score0);
  int NameValue(BirdScore *score0);
  static int(ScoreFile::*SCORE_VALUE_SELECT[SCORE_ELEMENT_NUM])
    (BirdScore *score0);
  int ValueSelect(BirdScore *score0,int element);

};

#endif
